#define PERL_constant_NOTFOUND	1
#define PERL_constant_NOTDEF	2
#define PERL_constant_ISIV	3
#define PERL_constant_ISNO	4
#define PERL_constant_ISNV	5
#define PERL_constant_ISPV	6
#define PERL_constant_ISPVN	7
#define PERL_constant_ISSV	8
#define PERL_constant_ISUNDEF	9
#define PERL_constant_ISUV	10
#define PERL_constant_ISYES	11

#ifndef NVTYPE
typedef double NV; /* 5.6 and later define NVTYPE, and typedef NV to it.  */
#endif
#ifndef aTHX_
#define aTHX_ /* 5.6 or later define this for threading support.  */
#endif
#ifndef pTHX_
#define pTHX_ /* 5.6 or later define this for threading support.  */
#endif

static HE *
constant_add_symbol(pTHX_  HV *hash, const char *name, I32 namelen, SV *value) {
    HE *he = (HE*) hv_common_key_len(hash, name, namelen, HV_FETCH_LVALUE, NULL,
				     0);
    SV *sv;

    if (!he) {
        croak("Couldn't add key '%s' to %%Ouroboros::",
		   name);
    }
    sv = HeVAL(he);
    if (SvOK(sv) || SvTYPE(sv) == SVt_PVGV) {
	/* Someone has been here before us - have to make a real sub.  */
	newCONSTSUB(hash, name, value);
    } else {
	SvUPGRADE(sv, SVt_RV);
	SvRV_set(sv, value);
	SvROK_on(sv);
	SvREADONLY_on(value);
    }
    return he;
}



#ifndef SYMBIAN

/* Store a hash of all symbols missing from the package. To avoid trampling on
   the package namespace (uninvited) put each package's hash in our namespace.
   To avoid creating lots of typeblogs and symbol tables for sub-packages, put
   each package's hash into one hash in our namespace.  */

static HV *
get_missing_hash(pTHX) {
    HV *const parent
	= get_hv("ExtUtils::Constant::ProxySubs::Missing", GVf_MULTI);
    /* We could make a hash of hashes directly, but this would confuse anything
	at Perl space that looks at us, and as we're visible in Perl space,
	best to play nice. */
    SV *const *const ref
	= hv_fetch(parent, "Ouroboros", 9, TRUE);
    HV *new_hv;

    if (!ref)
	return NULL;

    if (SvROK(*ref))
	return (HV*) SvRV(*ref);

    new_hv = newHV();
    SvUPGRADE(*ref, SVt_RV);
    SvRV_set(*ref, (SV *)new_hv);
    SvROK_on(*ref);
    return new_hv;
}

#endif

struct notfound_s {const char *name; I32 namelen;} ;

    static const struct notfound_s values_for_notfound[] =
      {
#ifndef SV_CATBYTES
        { "SV_CATBYTES", 11 },
#endif
#ifndef SV_CATUTF8
        { "SV_CATUTF8", 10 },
#endif
#ifndef SV_CONST_RETURN
        { "SV_CONST_RETURN", 15 },
#endif
#ifndef SV_COW_DROP_PV
        { "SV_COW_DROP_PV", 14 },
#endif
#ifndef SV_FORCE_UTF8_UPGRADE
        { "SV_FORCE_UTF8_UPGRADE", 21 },
#endif
#ifndef SV_GMAGIC
        { "SV_GMAGIC", 9 },
#endif
#ifndef SV_HAS_TRAILING_NUL
        { "SV_HAS_TRAILING_NUL", 19 },
#endif
#ifndef SV_IMMEDIATE_UNREF
        { "SV_IMMEDIATE_UNREF", 18 },
#endif
#ifndef SV_NOSTEAL
        { "SV_NOSTEAL", 10 },
#endif
#ifndef SV_SMAGIC
        { "SV_SMAGIC", 9 },
#endif
#ifndef GV_ADD
        { "GV_ADD", 6 },
#endif
#ifndef GV_ADDMG
        { "GV_ADDMG", 8 },
#endif
#ifndef GV_ADDMULTI
        { "GV_ADDMULTI", 11 },
#endif
#ifndef GV_NOADD_NOINIT
        { "GV_NOADD_NOINIT", 15 },
#endif
#ifndef GV_NOEXPAND
        { "GV_NOEXPAND", 11 },
#endif
#ifndef GV_NOINIT
        { "GV_NOINIT", 9 },
#endif
#ifndef GV_SUPER
        { "GV_SUPER", 8 },
#endif
#ifndef PERL_MAGIC_arylen
        { "PERL_MAGIC_arylen", 17 },
#endif
#ifndef PERL_MAGIC_arylen_p
        { "PERL_MAGIC_arylen_p", 19 },
#endif
#ifndef PERL_MAGIC_backref
        { "PERL_MAGIC_backref", 18 },
#endif
#ifndef PERL_MAGIC_bm
        { "PERL_MAGIC_bm", 13 },
#endif
#ifndef PERL_MAGIC_checkcall
        { "PERL_MAGIC_checkcall", 20 },
#endif
#ifndef PERL_MAGIC_collxfrm
        { "PERL_MAGIC_collxfrm", 19 },
#endif
#ifndef PERL_MAGIC_dbfile
        { "PERL_MAGIC_dbfile", 17 },
#endif
#ifndef PERL_MAGIC_dbline
        { "PERL_MAGIC_dbline", 17 },
#endif
#ifndef PERL_MAGIC_debugvar
        { "PERL_MAGIC_debugvar", 19 },
#endif
#ifndef PERL_MAGIC_defelem
        { "PERL_MAGIC_defelem", 18 },
#endif
#ifndef PERL_MAGIC_env
        { "PERL_MAGIC_env", 14 },
#endif
#ifndef PERL_MAGIC_envelem
        { "PERL_MAGIC_envelem", 18 },
#endif
#ifndef PERL_MAGIC_ext
        { "PERL_MAGIC_ext", 14 },
#endif
#ifndef PERL_MAGIC_fm
        { "PERL_MAGIC_fm", 13 },
#endif
#ifndef PERL_MAGIC_foo
        { "PERL_MAGIC_foo", 14 },
#endif
#ifndef PERL_MAGIC_hints
        { "PERL_MAGIC_hints", 16 },
#endif
#ifndef PERL_MAGIC_hintselem
        { "PERL_MAGIC_hintselem", 20 },
#endif
#ifndef PERL_MAGIC_isa
        { "PERL_MAGIC_isa", 14 },
#endif
#ifndef PERL_MAGIC_isaelem
        { "PERL_MAGIC_isaelem", 18 },
#endif
#ifndef PERL_MAGIC_lvref
        { "PERL_MAGIC_lvref", 16 },
#endif
#ifndef PERL_MAGIC_nkeys
        { "PERL_MAGIC_nkeys", 16 },
#endif
#ifndef PERL_MAGIC_overload_table
        { "PERL_MAGIC_overload_table", 25 },
#endif
#ifndef PERL_MAGIC_pos
        { "PERL_MAGIC_pos", 14 },
#endif
#ifndef PERL_MAGIC_qr
        { "PERL_MAGIC_qr", 13 },
#endif
#ifndef PERL_MAGIC_regdata
        { "PERL_MAGIC_regdata", 18 },
#endif
#ifndef PERL_MAGIC_regdatum
        { "PERL_MAGIC_regdatum", 19 },
#endif
#ifndef PERL_MAGIC_regex_global
        { "PERL_MAGIC_regex_global", 23 },
#endif
#ifndef PERL_MAGIC_rhash
        { "PERL_MAGIC_rhash", 16 },
#endif
#ifndef PERL_MAGIC_shared
        { "PERL_MAGIC_shared", 17 },
#endif
#ifndef PERL_MAGIC_shared_scalar
        { "PERL_MAGIC_shared_scalar", 24 },
#endif
#ifndef PERL_MAGIC_sig
        { "PERL_MAGIC_sig", 14 },
#endif
#ifndef PERL_MAGIC_sigelem
        { "PERL_MAGIC_sigelem", 18 },
#endif
#ifndef PERL_MAGIC_substr
        { "PERL_MAGIC_substr", 17 },
#endif
#ifndef PERL_MAGIC_sv
        { "PERL_MAGIC_sv", 13 },
#endif
#ifndef PERL_MAGIC_symtab
        { "PERL_MAGIC_symtab", 17 },
#endif
#ifndef PERL_MAGIC_taint
        { "PERL_MAGIC_taint", 16 },
#endif
#ifndef PERL_MAGIC_tied
        { "PERL_MAGIC_tied", 15 },
#endif
#ifndef PERL_MAGIC_tiedelem
        { "PERL_MAGIC_tiedelem", 19 },
#endif
#ifndef PERL_MAGIC_tiedscalar
        { "PERL_MAGIC_tiedscalar", 21 },
#endif
#ifndef PERL_MAGIC_utf8
        { "PERL_MAGIC_utf8", 15 },
#endif
#ifndef PERL_MAGIC_uvar
        { "PERL_MAGIC_uvar", 15 },
#endif
#ifndef PERL_MAGIC_uvar_elem
        { "PERL_MAGIC_uvar_elem", 20 },
#endif
#ifndef PERL_MAGIC_vec
        { "PERL_MAGIC_vec", 14 },
#endif
#ifndef PERL_MAGIC_vstring
        { "PERL_MAGIC_vstring", 18 },
#endif
        { NULL, 0 } };
struct iv_s {const char *name; I32 namelen; IV value;};
struct uv_s {const char *name; I32 namelen; UV value;};
